@@ -14,6 +14,7 @@ import matplotlib.gridspec as gridspec
 from typing import Union, Optional, List
 from typing_extensions import Literal
 from scanpy.plotting._utils import savefig_or_show
+import scanpy as sc
 
 from .. import logging as logg
 from ..tools.utils import getpath, importeR
@@ -23,7 +24,8 @@ from .dendrogram import dendrogram
 Rpy2, R, rstats, rmgcv, Formula = importeR("fitting associated features")
 check = [type(imp) == str for imp in [Rpy2, R, rstats, rmgcv, Formula]]
 
-from .modules import get_modules
+from ..get import modules as get_modules
+from .trajectory import remove_info
 from .utils import gen_milestones_gradients
 from .. import logging as logg
 
@@ -31,6 +33,7 @@ from .. import logging as logg
 def trends(
     adata: AnnData,
     features=None,
+    cluster=None,
     highlight_features: Union[List, Literal["A", "fdr"]] = "A",
     n_features: int = 10,
     root_milestone: Union[None, str] = None,
@@ -42,6 +45,9 @@ def trends(
     feature_cmap: str = "RdBu_r",
     pseudo_cmap: str = "viridis",
     plot_emb: bool = True,
+    plot_heatmap: bool = True,
+    wspace: Union[None, float] = None,
+    show_segs: bool = True,
     basis: str = "umap",
     heatmap_space: float = 0.5,
     offset_names: float = 0.15,
@@ -54,7 +60,8 @@ def trends(
     filter_complex: bool = False,
     complex_thre: float = 0.7,
     complex_z: float = 3,
-    fig_heigth: float = 4,
+    figsize: Union[None, tuple] = None,
+    axemb=None,
     show: Optional[bool] = None,
     output_mean: bool = False,
     save: Union[str, bool, None] = None,
@@ -93,6 +100,12 @@ def trends(
         colormap for pseudotime.
     plot_emb
         call pl.trajectory on the left side.
+    plot_heatmap
+        show heatmap on the right side.
+    wspace
+        width space between emb and heatmap.
+    show_segs
+        display segments on emb.
     basis
         Name of the `obsm` basis to use if plot_emb is True.
     heatmap_space
@@ -115,8 +128,10 @@ def trends(
         quantile to consider for complex filtering
     complex_z
         standard deviation threshold on the quantile subsetted feature.
-    fig_heigth
-        figure height.
+    figsize
+        figure size.
+    axemb
+        existing ax for plotting emb
     output_mean
         output mean fitted values to adata.
     show
@@ -184,8 +199,11 @@ def trends(
 
         adata = adata[cells]
 
-    if features is None:
+    if (features is None) & (cluster is None):
         features = adata.var_names
+    if cluster is not None:
+        features = adata.var_names[adata.var.fit_clusters == cluster]
+
     if branch is not None:
         name = root_milestone + "->" + "<>".join(milestones)
         df = adata.uns[name]["fork"]
@@ -305,163 +323,179 @@ def trends(
     else:
         annot = None
 
-    ratio = plt.rcParams["figure.figsize"][0] / plt.rcParams["figure.figsize"][1]
-    fig = plt.figure(figsize=(fig_heigth + (fig_heigth * plot_emb * ratio), fig_heigth))
-    outer = gridspec.GridSpec(1, 2, figure=fig, width_ratios=[1 * plot_emb * ratio, 1])
+    if axemb is None:
+        ratio = plt.rcParams["figure.figsize"][0] / plt.rcParams["figure.figsize"][1]
+        figsize = (
+            ((4 * plot_emb * ratio) + 4 * plot_heatmap, 4)
+            if figsize is None
+            else figsize
+        )
+        fig = plt.figure(figsize=figsize)
+        gsubs = gridspec.GridSpec(
+            1,
+            2,
+            figure=fig,
+            width_ratios=[1 * plot_emb * ratio, 1 * plot_heatmap],
+            wspace=wspace,
+        )
+        axs = []
+    if plot_heatmap:
+        gs_ht = gridspec.GridSpecFromSubplotSpec(
+            2 + (annot is not None),
+            1,
+            height_ratios=(1, 1, 18) if (annot is not None) else (1, 19),
+            subplot_spec=gsubs[1],
+            hspace=0,
+        )
+        i = 0
+        if annot is not None:
+            axannot = plt.subplot(gs_ht[i])
+            axs = axs + [axannot]
+            sns.heatmap(
+                pd.DataFrame(range(fitted_sorted.shape[1])).T,
+                robust=False,
+                rasterized=True,
+                cmap=annot_cmap[fitted_sorted.columns].values.tolist(),
+                xticklabels=False,
+                yticklabels=False,
+                cbar=False,
+                ax=axannot,
+            )
+            i = i + 1
 
-    gs_ht = gridspec.GridSpecFromSubplotSpec(
-        2 + (annot is not None),
-        1,
-        height_ratios=(1, 1, 18) if (annot is not None) else (1, 19),
-        subplot_spec=outer[1],
-        hspace=0,
-    )
-    axs = []
-    i = 0
-    if annot is not None:
-        axannot = plt.subplot(gs_ht[i])
-        axs = axs + [axannot]
+        axpsdt = plt.subplot(gs_ht[i])
+        axs = axs + [axpsdt]
         sns.heatmap(
-            pd.DataFrame(range(fitted_sorted.shape[1])).T,
-            robust=False,
+            pd.DataFrame(adata.obs.t[fitted_sorted.columns].values).T,
+            robust=True,
             rasterized=True,
-            cmap=annot_cmap[fitted_sorted.columns].values.tolist(),
+            cmap=pseudo_cmap,
             xticklabels=False,
             yticklabels=False,
             cbar=False,
-            ax=axannot,
+            ax=axpsdt,
+            vmax=adata.obs.t.max(),
         )
-        i = i + 1
 
-    axpsdt = plt.subplot(gs_ht[i])
-    axs = axs + [axpsdt]
-    sns.heatmap(
-        pd.DataFrame(adata.obs.t[fitted_sorted.columns].values).T,
-        robust=True,
-        rasterized=True,
-        cmap=pseudo_cmap,
-        xticklabels=False,
-        yticklabels=False,
-        cbar=False,
-        ax=axpsdt,
-        vmax=adata.obs.t.max(),
-    )
-
-    axheatmap = plt.subplot(gs_ht[i + 1])
-    axs = axs + [axheatmap]
-    sns.heatmap(
-        fitted_sorted,
-        robust=True,
-        cmap=feature_cmap,
-        rasterized=True,
-        xticklabels=False,
-        yticklabels=False,
-        ax=axheatmap,
-        cbar=False,
-    )
-
-    def add_frames(axis, vert):
-        rect = patches.Rectangle(
-            (0, 0),
-            len(fitted_sorted.columns),
-            vert,
-            linewidth=1,
-            edgecolor="k",
-            facecolor="none",
+        axheatmap = plt.subplot(gs_ht[i + 1])
+        axs = axs + [axheatmap]
+        sns.heatmap(
+            fitted_sorted,
+            robust=True,
+            cmap=feature_cmap,
+            rasterized=True,
+            xticklabels=False,
+            yticklabels=False,
+            ax=axheatmap,
+            cbar=False,
         )
-        # Add the patch to the Axes
-        axis.add_patch(rect)
-        offset = 0
-        for s in seg_order[:-1]:
-            prev_offset = offset
-            offset = offset + (adata.obs.seg == s).sum()
+
+        def add_frames(axis, vert):
             rect = patches.Rectangle(
-                (prev_offset, 0),
-                (adata.obs.seg == s).sum(),
+                (0, 0),
+                len(fitted_sorted.columns),
                 vert,
                 linewidth=1,
                 edgecolor="k",
                 facecolor="none",
             )
+            # Add the patch to the Axes
             axis.add_patch(rect)
-        return axis
+            offset = 0
+            for s in seg_order[:-1]:
+                prev_offset = offset
+                offset = offset + (adata.obs.seg == s).sum()
+                rect = patches.Rectangle(
+                    (prev_offset, 0),
+                    (adata.obs.seg == s).sum(),
+                    vert,
+                    linewidth=1,
+                    edgecolor="k",
+                    facecolor="none",
+                )
+                axis.add_patch(rect)
+            return axis
 
-    axpsdt = add_frames(axpsdt, 1)
+        axpsdt = add_frames(axpsdt, 1)
 
-    if annot is not None:
-        axannot = add_frames(axannot, 1)
+        if annot is not None:
+            axannot = add_frames(axannot, 1)
 
-    axheatmap = add_frames(axheatmap, fitted_sorted.shape[0])
+        axheatmap = add_frames(axheatmap, fitted_sorted.shape[0])
 
-    if highlight_features == "A":
-        highlight_features = (
-            adata.var.A[features].sort_values(ascending=False)[:n_features].index
-        )
-    elif highlight_features == "fdr":
-        highlight_features = (
-            adata.var.fdr[features].sort_values(ascending=True)[:n_features].index
-        )
-    xs = np.repeat(fitted_sorted.shape[1], len(highlight_features))
-    ys = (
-        np.array(
-            list(
-                map(
-                    lambda g: np.argwhere(fitted_sorted.index == g)[0][0],
-                    highlight_features,
+        if highlight_features == "A":
+            highlight_features = (
+                adata.var.A[features].sort_values(ascending=False)[:n_features].index
+            )
+        elif highlight_features == "fdr":
+            highlight_features = (
+                adata.var.fdr[features].sort_values(ascending=True)[:n_features].index
+            )
+        xs = np.repeat(fitted_sorted.shape[1], len(highlight_features))
+        ys = (
+            np.array(
+                list(
+                    map(
+                        lambda g: np.argwhere(fitted_sorted.index == g)[0][0],
+                        highlight_features,
+                    )
                 )
             )
+            + 0.5
         )
-        + 0.5
-    )
 
-    texts = []
-    for x, y, s in zip(xs, ys, highlight_features):
-        texts.append(axheatmap.text(x, y, s, fontsize=fontsize, style=style))
+        texts = []
+        for x, y, s in zip(xs, ys, highlight_features):
+            texts.append(axheatmap.text(x, y, s, fontsize=fontsize, style=style))
 
-    patch = patches.Rectangle(
-        (0, 0),
-        fitted_sorted.shape[1] + fitted_sorted.shape[1] * offset_names,
-        fitted_sorted.shape[0],
-        alpha=0,
-    )  # We add a rectangle to make sure the labels don't move to the right
-    axpsdt.set_xlim(
-        (0, fitted_sorted.shape[1] + fitted_sorted.shape[1] * offset_heatmap)
-    )
-    if annot is not None:
-        axannot.set_xlim(
+        patch = patches.Rectangle(
+            (0, 0),
+            fitted_sorted.shape[1] + fitted_sorted.shape[1] * offset_names,
+            fitted_sorted.shape[0],
+            alpha=0,
+        )  # We add a rectangle to make sure the labels don't move to the right
+        axpsdt.set_xlim(
             (0, fitted_sorted.shape[1] + fitted_sorted.shape[1] * offset_heatmap)
         )
-    axheatmap.set_xlim(
-        (0, fitted_sorted.shape[1] + fitted_sorted.shape[1] * offset_heatmap)
-    )
-    axheatmap.add_patch(patch)
-    axheatmap.hlines(
-        fitted_sorted.shape[0], 0, fitted_sorted.shape[1], color="k", clip_on=True
-    )
+        if annot is not None:
+            axannot.set_xlim(
+                (0, fitted_sorted.shape[1] + fitted_sorted.shape[1] * offset_heatmap)
+            )
+        axheatmap.set_xlim(
+            (0, fitted_sorted.shape[1] + fitted_sorted.shape[1] * offset_heatmap)
+        )
+        axheatmap.add_patch(patch)
+        axheatmap.hlines(
+            fitted_sorted.shape[0], 0, fitted_sorted.shape[1], color="k", clip_on=True
+        )
 
-    adjust_text(
-        texts,
-        ax=axheatmap,
-        add_objects=[patch],
-        va="center",
-        ha="left",
-        autoalign=False,
-        expand_text=(1.05, 1.2),
-        lim=5000,
-        only_move={"text": "y", "objects": "x"},
-        precision=0.1,
-        expand_points=(1.2, 1.05),
-    )
+        adjust_text(
+            texts,
+            ax=axheatmap,
+            add_objects=[patch],
+            va="center",
+            ha="left",
+            autoalign=False,
+            expand_text=(1.05, 1.2),
+            lim=5000,
+            only_move={"text": "y", "objects": "x"},
+            precision=0.1,
+            expand_points=(1.2, 1.05),
+        )
 
-    for i in range(len(xs)):
-        xx = [xs[i] + 1, fitted_sorted.shape[1] + fitted_sorted.shape[1] * offset_names]
-        yy = [ys[i], texts[i].get_position()[1]]
-        axheatmap.plot(xx, yy, color="k", linewidth=0.75)
+        for i in range(len(xs)):
+            xx = [
+                xs[i] + 1,
+                fitted_sorted.shape[1] + fitted_sorted.shape[1] * offset_names,
+            ]
+            yy = [ys[i], texts[i].get_position()[1]]
+            axheatmap.plot(xx, yy, color="k", linewidth=0.75)
 
     if plot_emb:
-        gs_emb = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=outer[0])
-        axemb = fig.add_subplot(gs_emb[0])
-        axs = axs + [axemb]
+        if axemb is None:
+            gs_emb = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gsubs[0])
+            axemb = fig.add_subplot(gs_emb[0])
+            axs = axs + [axemb]
         adata_temp.obs["mean_trajectory"] = 0
         adata_temp.obs.loc[
             fitted_sorted.columns, "mean_trajectory"
@@ -490,17 +524,32 @@ def trends(
                 show_info=False,
                 **kwargs,
             )
+        elif show_segs == False:
+            sc.pl.embedding(
+                adata=adata_temp,
+                basis=basis,
+                color=color,
+                cmap=feature_cmap,
+                ax=axemb,
+                title=title,
+                show=False,
+                **kwargs,
+            )
+            remove_info(adata_temp, axemb, color)
         else:
             trajectory(
                 adata=adata_temp,
                 basis=basis,
                 color_seg=color,
+                color_cells=color,
                 cmap_seg=feature_cmap,
+                cmap=feature_cmap,
                 show_info=False,
                 ax=axemb,
                 title=title,
                 root_milestone=root_milestone,
                 milestones=milestones,
+                show=False,
                 **kwargs,
             )
 
@@ -510,7 +559,7 @@ def trends(
                 f.write("%s\n" % item)
 
     if show == False:
-        return axs
+        return axs if plot_heatmap else axemb
     if save is not None:
         savefile = "figures/trends" + save
         logg.warn("saving figure to file " + savefile)
@@ -534,7 +583,10 @@ def single_trend(
     cmap_seg: str = "RdBu_r",
     cmap_cells: str = "RdBu_r",
     plot_emb: bool = True,
+    wspace: Union[None, float] = None,
     figsize: tuple = (8, 4),
+    ax_trend=None,
+    ax_emb=None,
     show: Optional[bool] = None,
     save: Union[str, bool, None] = None,
     **kwargs,
@@ -575,8 +627,16 @@ def single_trend(
         colormap for trajectory segments on left plot.
     cmap_cells
         colormap for cells on left plot.
+    plot_emb
+        plot the emb on the left side.
+    wspace
+        width space between emb and heatmap.
     figsize
-        figure size in inches
+        figure size in inches.
+    ax_trend
+        existing ax for trends, only works when emb plot is disabled.
+    ax_emb
+        existing ax for embedding plot.
     show
         show the plot.
     save
@@ -587,7 +647,6 @@ def single_trend(
     If `show==False` a tuple of two :class:`~matplotlib.axes.Axes`
 
     """
-
     if root_milestone is None:
         color_key = "seg_colors"
         if color_key not in adata.uns:
@@ -622,7 +681,9 @@ def single_trend(
         miles_cat = adata.obs.milestones.cat.categories
         mil_col = np.array(adata.uns["milestones_colors"])
 
-        X_early, X_late = get_modules(adata, root_milestone, milestones, layer)
+        X_modules = get_modules(adata, root_milestone, milestones, layer)
+        X_early, X_late = X_modules.iloc[:, :2], X_modules.iloc[:, 2:]
+
         X = pd.concat([X_early, X_late], axis=1)
         X["t"] = adata.obs.loc[X.index, "t"].values
 
@@ -646,16 +707,22 @@ def single_trend(
         ).sort_values("t")
 
         color_exp = mil_col[miles_cat == branch][0]
-
-    if plot_emb:
-        fig, axs = plt.subplots(1, 2, figsize=figsize)
-    else:
-        fig, axs = plt.subplots(1, 1, figsize=figsize)
-        axs = ["empty", axs]
+    ratio = plt.rcParams["figure.figsize"][0] / plt.rcParams["figure.figsize"][1]
+    if (ax_emb is None) & (ax_trend is None):
+        if plot_emb:
+            fig, (ax_emb, ax_trend) = plt.subplots(
+                1,
+                2,
+                figsize=figsize,
+                gridspec_kw=dict(width_ratios=[1 * ratio, 1], wspace=wspace),
+            )
+        else:
+            fig, ax_trend = plt.subplots(1, 1, figsize=figsize)
+            # axs = ["empty", axs]
 
     for s in df.seg.unique():
         if color_exp is None:
-            axs[1].scatter(
+            ax_trend.scatter(
                 df.loc[df.seg == s, "t"],
                 df.loc[df.seg == s, "expression"],
                 alpha=alpha_expr,
@@ -664,7 +731,7 @@ def single_trend(
                     np.argwhere(adata.obs.seg.cat.categories == s)[0][0]
                 ],
             )
-            axs[1].plot(
+            ax_trend.plot(
                 df.loc[df.seg == s, "t"],
                 df.loc[df.seg == s, "fitted"],
                 c=adata.uns["seg_colors"][
@@ -678,7 +745,7 @@ def single_trend(
                     adata.uns["graph"]["pp_seg"].loc[:, "from"].isin([tolink]).values
                 ).flatten()
             ]:
-                axs[1].plot(
+                ax_trend.plot(
                     [
                         df.loc[df.seg == s, "t"].iloc[-1],
                         df.loc[df.seg == next_s, "t"].iloc[0],
@@ -693,14 +760,14 @@ def single_trend(
                     linewidth=fitted_linewidth,
                 )
         else:
-            axs[1].scatter(
+            ax_trend.scatter(
                 df.loc[df.seg == s, "t"],
                 df.loc[df.seg == s, "expression"],
                 c=color_exp,
                 alpha=alpha_expr,
                 s=size_expr,
             )
-            axs[1].plot(
+            ax_trend.plot(
                 df.loc[df.seg == s, "t"],
                 df.loc[df.seg == s, "fitted"],
                 c=color_exp,
@@ -712,7 +779,7 @@ def single_trend(
                     adata.uns["graph"]["pp_seg"].loc[:, "from"].isin([tolink]).values
                 ).flatten()
             ]:
-                axs[1].plot(
+                ax_trend.plot(
                     [
                         df.loc[df.seg == s, "t"].iloc[-1],
                         df.loc[df.seg == next_s, "t"].iloc[0],
@@ -725,31 +792,45 @@ def single_trend(
                     linewidth=fitted_linewidth,
                 )
 
-    axs[1].set_ylabel(ylab)
-    axs[1].set_xlabel("pseudotime")
-    x0, x1 = axs[1].get_xlim()
-    y0, y1 = axs[1].get_ylim()
+    ax_trend.set_ylabel(ylab)
+    ax_trend.set_xlabel("pseudotime")
+    x0, x1 = ax_trend.get_xlim()
+    y0, y1 = ax_trend.get_ylim()
     if plot_emb:
-        axs[1].set_aspect(abs(x1 - x0) / abs(y1 - y0))
+        ax_trend.set_aspect(abs(x1 - x0) / abs(y1 - y0))
 
     if plot_emb:
-        trajectory(
-            adata,
-            basis=basis,
-            color_seg=feature,
-            cmap_seg=cmap_seg,
-            color_cells=feature,
-            cmap=cmap_cells,
-            show_info=False,
-            ax=axs[0],
-            title=feature,
-            layer="fitted",
-            show=False,
-            save=False,
-            **kwargs,
-        )
+        if basis == "dendro":
+            dendrogram(
+                adata,
+                color=feature,
+                cmap=cmap_cells,
+                ax=ax_emb,
+                title=feature,
+                show_info=False,
+                layer="fitted",
+                show=False,
+                save=False,
+                **kwargs,
+            )
+        else:
+            trajectory(
+                adata,
+                basis=basis,
+                color_seg=feature,
+                cmap_seg=cmap_seg,
+                color_cells=feature,
+                cmap=cmap_cells,
+                show_info=False,
+                ax=ax_emb,
+                title=feature,
+                layer="fitted",
+                show=False,
+                save=False,
+                **kwargs,
+            )
 
     savefig_or_show("single_trend", show=show, save=save)
 
     if show is False:
-        return axs if plot_emb else axs[1]
+        return (ax_emb, ax_trend) if plot_emb else ax_trend
