@@ -27,6 +27,9 @@ def graph(
     adata: AnnData,
     basis: str = "umap",
     size_nodes: float = None,
+    alpha_nodes: float = 1,
+    linewidth: float = 2,
+    alpha_seg: float = 1,
     color_cells: Union[str, None] = None,
     tips: bool = True,
     forks: bool = True,
@@ -47,7 +50,13 @@ def graph(
     basis
         Name of the `obsm` basis to use.
     size_nodes
-        size of the projected prinicpal points.
+        Size of the projected prinicpal points.
+    alpha_nodes
+        Alpha of nodes.
+    linewidth
+        Line width of the segments.
+    alpha_seg
+        Alpha of segments.
     color_cells
         cells color
     tips
@@ -108,10 +117,14 @@ def graph(
     segs = al.tolist()
     vertices = proj.tolist()
     lines = [[tuple(vertices[j]) for j in i] for i in segs]
-    lc = matplotlib.collections.LineCollection(lines, colors="k", linewidths=2)
+    lc = matplotlib.collections.LineCollection(
+        lines, colors="k", linewidths=linewidth, alpha=alpha_seg, rasterized=True
+    )
     ax.add_collection(lc)
 
-    ax.scatter(proj[:, 0], proj[:, 1], s=size_nodes, c="k")
+    ax.scatter(
+        proj[:, 0], proj[:, 1], s=size_nodes, c="k", alpha=alpha_nodes, rasterized=True
+    )
 
     bbox = dict(facecolor="white", alpha=0.6, edgecolor="white", pad=0.1)
 
@@ -309,17 +322,7 @@ def trajectory(
 
     anndata_logger.level = prelog
     if show_info == False and color_cells is not None:
-        if is_categorical(adata, color_cells):
-            if ax.get_legend() is not None:
-                ax.get_legend().remove()
-        else:
-            ax.set_box_aspect(aspect=1)
-            fig = ax.get_gridspec().figure
-            cbar = np.argwhere(
-                ["colorbar" in a.get_label() for a in fig.get_axes()]
-            ).ravel()
-            if len(cbar) > 0:
-                fig.get_axes()[cbar[0]].remove()
+        remove_info(adata, ax, color_cells)
 
     al = np.array(g.get_edgelist())
 
@@ -725,3 +728,17 @@ def _get_color_values(
             # that are not in the groups
             color_vector[~adata.obs[value_to_plot].isin(groups)] = "lightgray"
         return color_vector, True
+
+
+def remove_info(adata, ax, color):
+    if is_categorical(adata, color):
+        if ax.get_legend() is not None:
+            ax.get_legend().remove()
+    else:
+        ax.set_box_aspect(aspect=1)
+        fig = ax.get_gridspec().figure
+        cbar = np.argwhere(
+            ["colorbar" in a.get_label() for a in fig.get_axes()]
+        ).ravel()
+        if len(cbar) > 0:
+            fig.get_axes()[cbar[0]].remove()
